package com.zoniot.ccrc.commom.utils;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description:JWT工具类
 */
@Component
public class JwtTokenUtil implements Serializable {
    private static final String CLAIM_KEY_USERID = "sub";
    private static final String CLAIM_KEY_CREATED = "created";
    private static final String CLAIM_KEY_LOGIN_TOKTN = "loginToken";
    private static final long serialVersionUID = -8305152446124853696L;

    @Autowired
    RedisUtil redisUtil;

    /**
     * 密钥
     */
    @Value("smartCity123")
    private String secret;
    /**
     * //有效期
     */
    @Value("36000")
    private Long expiration;


    /**
     * 从数据声明生成令牌
     *
     * @param claims 数据声明
     * @return 令牌
     */
    private String generateToken(Map<String, Object> claims) {
        Date expirationDate = new Date(System.currentTimeMillis() + expiration * 1000);
        return Jwts.builder().setClaims(claims).setExpiration(expirationDate).signWith(SignatureAlgorithm.HS512, secret).compact();
    }

    /**
     * 将token存储到redis
     */
    public void setExpire(String key, String val, long time) {
        redisUtil.setExpire(key, val, time);
    }

    /**
     * 移除
     */
    public void del(String key) {
        redisUtil.del(key);
    }

    /**
     * 判断是否有效
     * @param authToken
     * @return
     */
    public Boolean validateToken(String authToken) {
        Object o = redisUtil.get(authToken);
        if(null != o){
            return true;
        }
        return false;
    }

    /**
     * 从令牌中获取数据声明
     *
     * @param token 令牌
     * @return 数据声明
     */
    private Claims getClaimsFromToken(String token) {
        Claims claims;
        try {
            claims = Jwts.parser().setSigningKey(secret).parseClaimsJws(token).getBody();
        } catch (Exception e) {
            claims = null;
        }
        return claims;
    }

    /**
     * 生成令牌
     *
     * @return 令牌
     */
    public String generateToken(Integer id, String loginToken) {
        Map<String, Object> claims = new HashMap<>();
        claims.put(CLAIM_KEY_USERID, id);
        claims.put(CLAIM_KEY_LOGIN_TOKTN, loginToken);
        return generateToken(claims);
    }

    /**
     * 从令牌中获取用户id
     *
     * @param token 令牌
     * @return 用户名
     */
    public Integer getUserIdFromToken(String token) {
        Integer userId;
        try {
            Claims claims = getClaimsFromToken(token);
            //userId = claims.getSubject();
            userId = claims.get(CLAIM_KEY_USERID, Integer.class);
        } catch (Exception e) {
            userId = null;
        }
        return userId;
    }

    /**
     * 判断令牌是否过期
     *
     * @param token 令牌
     * @return 是否过期
     */
    public Boolean isTokenExpired(String token) {
        try {
            Claims claims = getClaimsFromToken(token);
            Date expiration = claims.getExpiration();
            return expiration.before(new Date());
        } catch (Exception e) {
            return false;
        }
    }


    public String getLoginUserToken(String token){
        String loginUserToken;
        try {
            Claims claims = getClaimsFromToken(token);
            loginUserToken = claims.get(CLAIM_KEY_LOGIN_TOKTN, String.class);
        } catch (Exception e) {
            loginUserToken = null;
        }
        return loginUserToken;
    }

    /**
     * 验证令牌
     *
     * @param token       令牌
     * @return 是否有效
     */
    public Boolean validateToken(String token, Integer id, String loginToken) {
        Integer userId = getUserIdFromToken(token);
        String loginUserToken = getLoginUserToken(token);
        return (userId.equals(id) && !isTokenExpired(token) && loginToken.equals(loginUserToken));
    }

}

